@@ -21,10 +21,12 @@ sub- stanzas) handling routines
 from __future__ import annotations
 
 from typing import Any
+from typing import Iterable
 from typing import Union
 from typing import Optional
 from typing import cast
 
+import os
 import time
 import hashlib
 import functools
@@ -36,13 +38,18 @@ from dataclasses import asdict
 from gi.repository import GLib
 
 import idna
-from precis_i18n import get_profile
 from nbxmpp.simplexml import Node
 from nbxmpp.namespaces import Namespace
+from nbxmpp.stringprep import nodeprep
+from nbxmpp.stringprep import resourceprep
+from nbxmpp.precis import enforce_precis_username
+from nbxmpp.precis import enforce_precis_opaque
+
 
 def ascii_upper(s):
     return s.upper()
 
+
 SASL_AUTH_MECHS = [
     'SCRAM-SHA-512-PLUS',
     'SCRAM-SHA-512',
@@ -251,7 +258,6 @@ _errorcodes = {
 }
 
 
-_localpart_disallowed_chars = set('"&\'/:<>@')
 _localpart_escape_chars = ' "&\'/:<>@'
 
 
@@ -482,12 +488,17 @@ def validate_localpart(localpart: str) -> str:
     if not localpart or len(localpart.encode()) > 1023:
         raise LocalpartByteLimit
 
-    if _localpart_disallowed_chars & set(localpart):
-        raise LocalpartNotAllowedChar
+    if os.environ.get('NBXMPP_ENFORCE_PRECIS') is None:
+        try:
+            return nodeprep(localpart)
+        except Exception:
+            try:
+                return enforce_precis_username(localpart)
+            except Exception:
+                raise LocalpartNotAllowedChar
 
     try:
-        username = get_profile('UsernameCaseMapped')
-        return username.enforce(localpart)
+        return enforce_precis_username(localpart)
     except Exception:
         raise LocalpartNotAllowedChar
 
@@ -497,9 +508,17 @@ def validate_resourcepart(resourcepart: str) -> str:
     if not resourcepart or len(resourcepart.encode()) > 1023:
         raise ResourcepartByteLimit
 
+    if os.environ.get('NBXMPP_ENFORCE_PRECIS') is None:
+        try:
+            return resourceprep(resourcepart)
+        except Exception:
+            try:
+                return enforce_precis_opaque(resourcepart)
+            except Exception:
+                raise ResourcepartNotAllowedChar
+
     try:
-        opaque = get_profile('OpaqueString')
-        return opaque.enforce(resourcepart)
+        return enforce_precis_opaque(resourcepart)
     except Exception:
         raise ResourcepartNotAllowedChar
 
@@ -1254,6 +1273,27 @@ class Message(Protocol):
     def setHint(self, hint):
         self.setTag(hint, namespace=Namespace.HINTS)
 
+    def setReactions(self, target_id: str, emojis: Iterable[str]):
+        reactions = self.addChild(
+            'reactions', namespace=Namespace.REACTIONS, attrs={"id": target_id})
+        for e in emojis:
+            reactions.addChild(
+                'reaction', namespace=Namespace.REACTIONS, payload=[e])
+
+    def getReactions(self) -> Optional[tuple[str, set[str]]]:
+        reactions = self.getTag('reactions', namespace=Namespace.REACTIONS)
+        if not reactions:
+            return None
+
+        react_to = reactions.getAttr('id')
+        if not react_to:
+            return None
+
+        tags = reactions.getTags('reaction', namespace=Namespace.REACTIONS)
+
+        # strip() in case clients surround emojis with whitespace
+        return react_to, {t.getData().strip() for t in tags}
+
 
 class Presence(Protocol):
 
