@@ -30,8 +30,8 @@ from dataclasses import dataclass
 from dataclasses import field
 from datetime import datetime
 
-from gi.repository import Soup
 from gi.repository import Gio
+from gi.repository import GLib
 
 from nbxmpp.simplexml import Node
 from nbxmpp.namespaces import Namespace
@@ -708,8 +708,9 @@ class ProxyData(NamedTuple):
 
     def get_uri(self) -> str:
         if self.username is not None:
-            user_pass = Soup.uri_encode('%s:%s' % (self.username,
-                                                   self.password))
+            username = GLib.uri_escape_string(self.username, None, False)
+            password = GLib.uri_escape_string(self.password, None, False)
+            user_pass = f'{username}:{password}'
             return '%s://%s@%s' % (self.type,
                                    user_pass,
                                    self.host)
@@ -746,6 +747,11 @@ class ChatMarker(NamedTuple):
         return self.type == 'acknowledged'
 
 
+class Reactions(NamedTuple):
+    id: str
+    emojis: set[str]
+
+
 class CommonError:
     def __init__(self, stanza):
         self._stanza_name = stanza.getName()
@@ -993,6 +999,7 @@ class MessageProperties:
     xhtml: Optional[str] = None
     security_label = None
     chatstate = None
+    reactions: Optional[Reactions] = None
 
     def is_from_us(self, bare_match: bool = True):
         if self.from_ is None:
